import type { App } from 'vue'
import { createPinia } from 'pinia'
import Vant from 'vant'
import 'vant/lib/index.css'
import $ from 'jquery'

import './styles/all.less'
import * as AllComponents from './components/index'
import * as AllIcons from './icons/all'
import * as AllUtils from './utils/all'

AllUtils.config.routerSetting.layoutComponment = () => import('./components/layout/index.vue')

const installMethod = (app: App) => {
  app.use(createPinia())
  // 全局注册Vant组件
  app.use(Vant)
  //AllComponents
  for (const [key, component] of Object.entries(AllComponents)) {
    app.component(component.name ? component.name : key, component)
  }
  //AllIcons
  for (const [key, component] of Object.entries(AllIcons)) {
    app.component(component.name ? component.name : key, component)
  }
  //router路由
  if (AllUtils.router.routerPlug != null) {
    app.use(AllUtils.router.routerPlug)
  }
  //全局变量(windows)
  if (typeof window != 'undefined') {
    Object.assign(window, { app: app, zfz: AllUtils })
  }
  //全局变量(vue属性)
  app.config.globalProperties.$zfz = AllUtils
}

//自动计算page-body
window.setInterval(() => {
  $('.zfz-page-body').each(function (_, element) {
    if ($(element).hasClass('fill-height') === true) {
      let fillHeightValue = $(this).parents(".el-dialog").length > 0 ? parseInt($(element).parent().css('max-height').replace('px', '')) : $(element).parent().height() ?? 0
      let fillHeightByTop = 0
      let fillHeightAdded = 0
      const childrenElement = $(element).parent().children()
      let childrenIsBodyNow = false
      $(childrenElement).each(function (_, childElement) {
        if ($(childElement).hasClass('zfz-page-body') === true) {
          childrenIsBodyNow = true
        } else {
          fillHeightValue -= $(childElement).height() ?? 0
          fillHeightAdded += $(childElement).height() ?? 0
        }
        if (childrenIsBodyNow === false) {
          fillHeightByTop += $(childElement).height() ?? 0
        }
      })
      $(element).height(fillHeightValue + 'px')
      const lhsContainer = $(element).parent().find(".zfz-list-high-search-container");
      if (lhsContainer.length > 0) {
        lhsContainer.children(".background").css('top', fillHeightByTop + 'px').height('calc(100% - ' + fillHeightAdded + 'px)')
        lhsContainer.children(".contentBox").css('top', fillHeightByTop + 'px')
        lhsContainer.children(".contentBox").children(".search-body").css('max-height', (fillHeightValue - 130) + 'px')
      }
    }
  })
  $(".el-dialog__body").each(function (_, element) {
    $(element).css('max-height', (($(window).height() ?? 0) - 160) + 'px').css('overflow', 'auto')
  })
}, 50)

export const version = '1.1'
export const install = installMethod
export const zfz = AllUtils
export const jquery = $
export default installMethod
