import { ref, createApp, type Component } from 'vue'
import $ from 'jquery'
import { router, caches } from './router'
import { type RouteLocationRaw } from 'vue-router'
import * as ZfzUI from '@/lib/index'
import loading from './loading'
import message from './message'

const commandIndex = ref(1000)

//跳转指令:保留本页缓存，并跳转
export const navigateTo = function (params: RouteLocationRaw) {
    if (router.routerPlug) {
        const matched = router.routerPlug.currentRoute.value.matched
        const addName = matched ? matched[matched.length - 1].components?.default.name : ''
        if (addName) {
            caches.value.push(addName)
        }
    }
    router.routerPlug?.push(params)
}
//跳转指令:去除本页缓存，并跳转
export const redirectTo = function (params: RouteLocationRaw, replace: boolean = false) {
    if (router.routerPlug) {
        const matched = router.routerPlug.currentRoute.value.matched
        const addName = matched ? matched[matched.length - 1].components?.default.name : ''
        if (addName) {
            const myIndex = caches.value.indexOf(addName)
            if (myIndex > -1) {
                caches.value.splice(myIndex, 1)
            }
        }
    }
    if (replace) {
        router.routerPlug?.replace(params)
    } else {
        router.routerPlug?.push(params)
    }

}
//跳转指令:清除所有缓存，并跳转
export const reLaunch = function (params: RouteLocationRaw) {
    if (caches.value.length > 0) {
        caches.value.splice(0, caches.value.length)
    }
    router.routerPlug?.push(params)
}
//新vue指令:新建一个body下的vue app
export const newVueApp = (rootComponent: Component | Function, rootProps?: any, callback?: Function) => {
    commandIndex.value++
    const divIdValue = "zfz-vueapp-container-" + commandIndex.value;
    const divElement = $("<div id=\"" + divIdValue + "\"></div>")
    $(document.body).append(divElement);
    let appElement: any = undefined
    if (rootComponent instanceof Function) {
        const loadingItem = loading.page();
        (rootComponent as any)().then((res: any) => {
            appElement = createApp(res.default, rootProps)
            appElement?.use(ZfzUI.default)
            appElement?.provide("newVueApp.callback", callback)
            appElement?.provide('newVueApp.destroy', () => {
                appElement?.unmount()
                $('#' + divIdValue).remove();
            });
            appElement?.mount("#" + divIdValue)
            loadingItem.close()
        }).catch((err: any) => {
            message.toast(err)
            loadingItem.close()
        })
    } else {
        appElement = createApp(rootComponent, rootProps)
        appElement?.use(ZfzUI.default)
        appElement?.provide("newVueApp.callback", callback)
        appElement?.provide('newVueApp.destroy', () => {
            appElement?.unmount()
            $('#' + divIdValue).remove();
        });
        appElement?.mount("#" + divIdValue)
    }
}
//新vue指令:打开弹窗
export const openDialog = function (rootComponent: Component | Function, rootProps?: any, callback?: Function) {
    newVueApp(rootComponent, rootProps, callback)
}