import axios from 'axios'
import config from './config'
import { navigateTo } from './directive'
import { setAccessTokenFn } from './store'
import { useFrameStore } from './store'

//创建axios实例
const service = axios.create({
  baseURL: '',
  timeout: 30000
})
//request拦截器
service.interceptors.request.use(
  (reqCfg) => {
    //是否需要设置accessToken
    const store = useFrameStore()
    if (store.accessToken) {
      const requestKeyStr = config.loginSetting.accessToken.requestKeyStr;
      const requestPrefix = config.loginSetting.accessToken.requestPrefix;
      switch (config.loginSetting.accessToken.requestMethod) {
        case 'header':
          reqCfg.headers[requestKeyStr] = requestPrefix + store.accessToken
          break
        case 'querystring':
          reqCfg.url += reqCfg.url && reqCfg.url.indexOf('?') > 0 ? '&' : '?'
          reqCfg.url += requestKeyStr + '=' + requestPrefix + store.accessToken
          break
      }
    }
    config.requestConfig(reqCfg)
    return reqCfg
  },
  (error) => {
    //请求错误处理
    console.log('axios request error')
    console.log(error)
    return Promise.reject(error)
  }
)
//response拦截器
service.interceptors.response.use(
  (response) => {
    if (response.headers['access-token']) {
      setAccessTokenFn(response.headers['access-token'])
    }
    if (response.headers['not-logged-in']) {
      if (response.data && response.data.msg) {
        response.data.msg = '您尚未登录或登录已超时';
      }
      if (response.headers['not-logged-in'] == 'auto-redirect') {
        navigateTo({
          path: config.loginSetting.loginPageUrl + '?' + config.loginSetting.redirectName + '='
        })
      }
    }
    return response.data
  },
  (error) => {
    return Promise.resolve({
      code: 1,
      mark: 'AxiosError',
      msg: '接口请求失败:' + error.message
    })
  }
)

const hasHttpPrefix = function (urlStr: String) {
  const preValues = ['http://', 'https://']
  for (let i = 0; i < preValues.length; i++) {
    const pre_item = urlStr.substring(0, preValues[i].length)
    if (pre_item.toLocaleLowerCase() == preValues[i]) {
      return true
    }
  }
  return false
}
const requestMethod = function (params: any, requestType: string) {
  let params_url = params.url
  if (params_url && !hasHttpPrefix(params_url)) {
    params_url = config.apiPrefix + params_url
  }
  return service({
    url: params_url,
    headers: params.headers,
    method: requestType,
    data: params.data
  })
}

export const request = {
  get: function (params: any) {
    return requestMethod(params, 'get')
  },
  post: function (params: any) {
    return requestMethod(params, 'post')
  },
  put: function (params: any) {
    return requestMethod(params, 'put')
  },
  delete: function (params: any) {
    return requestMethod(params, 'delete')
  },
  head: function (params: any) {
    return requestMethod(params, 'head')
  },
  options: function (params: any) {
    return requestMethod(params, 'options')
  },
  method: function (params: any) {
    return requestMethod(params, params.type)
  }
}

export default request
