import { ref } from 'vue'
import { createRouter, createWebHistory, type Router } from 'vue-router'
import config from './config'
import { useFrameStore } from './store'
import { type FrameRoute, type SystemMenu } from '@/lib/types/all'
import loading from './loading'

const buildRoutesBySystemMenus = (routerPlug: Router | null, systemMenus: SystemMenu[], isAddLayout: boolean = false) => {
  if (isAddLayout) {
    routerPlug?.addRoute({
      path: '/',
      name: 'layout',
      component: config.routerSetting.layoutComponment,
      redirect: config.homePageUrl,
      children: []
    })
  }
  systemMenus.forEach((element) => {
    if (element.type === 'Directory' && element.children && element.children.length > 0) {
      buildRoutesBySystemMenus(routerPlug, element.children)
    } else {
      if (element.type === 'RoutePage') {
        const elemItem = { ...element }
        if (elemItem.keepAlive === undefined) {
          elemItem.keepAlive = true;
        }
        routerPlug?.addRoute('layout', {
          path: element.visitPath,
          name: element.code,
          component: () => {
            console.log("abc", config.srcViews)
            const compoResult = config.srcViews[`/src/views${element.compoPath}.vue`]
            const compoNewName = 'sysmenu-' + element.id
            return compoResult().then((comp: any) => ({
              ...comp.default,
              name: compoNewName
            }))
          },
          meta: elemItem
        })
      }
    }
  })
}

export const router: FrameRoute = {
  routerPlug: null,
  initialize: function () {
    if (this.routerPlug === null) {
      config.routerSetting.staticRoutes.forEach((item: any) => {
        item.staticRoute = true
      })
      this.routerPlug = createRouter({
        history: createWebHistory(config.baseUrlStr),
        routes: config.routerSetting.staticRoutes
      })
    }
    let loadingItem = null as any
    this.routerPlug.beforeEach((to, _from, next) => {
      if (to.meta && to.meta.keepAlive === false) {
        to.meta.keepAlive = true
      }
      loadingItem = loading.page()
      const store = useFrameStore()
      if (to.meta.loginFirst === true && !store.accessToken) {
        next(`${config.loginSetting.loginPageUrl}?redirect=${to.fullPath}`)
      } else if ((to as any).staticRoute === true) {
        next()
      } else {
        store.setSystemMenus((systemMenus: SystemMenu[], isAlready: Boolean) => {
          if (isAlready) {
            next()
          } else {
            buildRoutesBySystemMenus(this.routerPlug, systemMenus, true)
            next({ ...to, replace: true })
          }
        })
      }
    })
    this.routerPlug.afterEach(() => {
      loadingItem.close()
    })
  }
}

export const caches = ref([] as Array<string>)

export default router
