import { defineStore } from 'pinia'
import config from './config'
import message from './message'
import { type SystemMenu } from '@/lib/types/all'
import { useCookies } from 'vue3-cookies'

const { cookies } = useCookies()

export const getAccessTokenFn = (): string => {
  if (config.loginSetting.accessToken.storageMethod === 'cookie') {
    return cookies.get(config.loginSetting.accessToken.storageKeyStr)
  }
  if (config.loginSetting.accessToken.storageMethod === 'localStorage') {
    return window.localStorage.getItem(config.loginSetting.accessToken.storageKeyStr) ?? ''
  }
  if (config.loginSetting.accessToken.storageMethod === 'sessionStorage') {
    return window.sessionStorage.getItem(config.loginSetting.accessToken.storageKeyStr) ?? ''
  }
  return ''
}
export const setAccessTokenFn = (value: string) => {
  if (value) {
    if (config.loginSetting.accessToken.storageMethod === 'cookie') {
      cookies.set(
        config.loginSetting.accessToken.storageKeyStr,
        value,
        config.loginSetting.accessToken.storageExpire
      )
    }
    if (config.loginSetting.accessToken.storageMethod === 'localStorage') {
      window.localStorage.setItem(config.loginSetting.accessToken.storageKeyStr, value)
    }
    if (config.loginSetting.accessToken.storageMethod === 'sessionStorage') {
      window.sessionStorage.setItem(config.loginSetting.accessToken.storageKeyStr, value)
    }
  } else {
    if (config.loginSetting.accessToken.storageMethod === 'cookie') {
      return cookies.remove(config.loginSetting.accessToken.storageKeyStr)
    }
    if (config.loginSetting.accessToken.storageMethod === 'localStorage') {
      window.localStorage.removeItem(config.loginSetting.accessToken.storageKeyStr)
    }
    if (config.loginSetting.accessToken.storageMethod === 'sessionStorage') {
      window.sessionStorage.removeItem(config.loginSetting.accessToken.storageKeyStr)
    }
  }
}

export const useFrameStore = defineStore({
  id: 'frame',
  state: () => {
    return {
      accessToken: getAccessTokenFn(),
      currentUser: config.loginSetting.getCurrentUser(),
      systemMenus: [] as SystemMenu[],
      systemMenusIsAlready: false as Boolean
    }
  },
  getters: {},
  actions: {
    setAccessToken(value: string) {
      this.accessToken = value
      setAccessTokenFn(value)
    },
    setSystemMenus(callback: Function) {
      if (this.systemMenusIsAlready) {
        callback(this.$state.systemMenus, true)
      } else {
        config.routerSetting.dynamicMenus().then((res: any) => {
          if (res.code == 0) {
            this.systemMenus = config.routerSetting.formatMenus(res.data.rows)
          }
          this.systemMenusIsAlready = true
          if (res.code === 0) {
            callback(res.data.rows, false)
          } else {
            message.toast({
              message: res.msg,
              type: res.code
            })
          }
        })
      }
    }
  }
})
